@@ -22,6 +22,10 @@ args = argparser.parse_args()
 newenv = os.environ.copy()
 newenv['GDK_PIXBUF_PIXDATA'] = args.pixdata
 newenv['GDK_PIXBUF_MODULE_FILE'] = args.loaders
+# 'nt': NT-based Windows, see https://docs.python.org/3/library/os.html
+if os.name == 'nt':
+    gdk_pixbuf_dll_buildpath = os.path.dirname(args.pixdata)
+    newenv['PATH'] = gdk_pixbuf_dll_buildpath + os.pathsep + newenv['PATH']
 
 cmd = args.printer
 
