@@ -25,6 +25,7 @@
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
+#include <string.h>
 
 #include <gio/gio.h>
 #include <errno.h>
@@ -127,6 +128,7 @@ timeval_to_usec (const GTimeVal *timeval)
 typedef struct {
   GdkPixbuf *pixbuf;
   gint64 delay; // usec
+  gboolean is_last_frame;
 } GdkPixbufGlycinFrame;
 
 static void
@@ -156,7 +158,6 @@ struct _GdkPixbufGlycinAnimationIter
   GdkPixbufGlycinAnimation *animation;
   guint idx;
   gint64 time;
-  gboolean on_last_frame;
 };
 
 G_DEFINE_FINAL_TYPE (GdkPixbufGlycinAnimation, gdk_pixbuf_glycin_animation, GDK_TYPE_PIXBUF_ANIMATION)
@@ -216,7 +217,6 @@ gdk_pixbuf_glycin_animation_get_iter (GdkPixbufAnimation *animation,
       iter->animation = g_object_ref (self);
       iter->idx = 0;
       iter->time = timeval_to_usec (start_time);
-      iter->on_last_frame = FALSE;
     }
   else
     iter = NULL;
@@ -264,9 +264,6 @@ gdk_pixbuf_glycin_animation_iter_get_delay_time (GdkPixbufAnimationIter *iter)
   GdkPixbufGlycinAnimationIter *self = (GdkPixbufGlycinAnimationIter *) iter;
   int delay_time;
 
-  if (self->on_last_frame)
-    return -1;
-
   delay_time = g_array_index (self->animation->decoded, GdkPixbufGlycinFrame, self->idx).delay / 1000; /* usec to millis */
 
   return delay_time;
@@ -293,38 +290,68 @@ gdk_pixbuf_glycin_animation_iter_advance (GdkPixbufAnimationIter *iter,
 {
   GdkPixbufGlycinAnimationIter *self = (GdkPixbufGlycinAnimationIter *) iter;
   gint64 new_time;
-  gboolean retval;
-
-  if (self->on_last_frame)
-    return FALSE;
 
   new_time = timeval_to_usec (current_time);
 
-  retval = FALSE;
   while ((self->time + g_array_index (self->animation->decoded, GdkPixbufGlycinFrame, self->idx).delay) < new_time)
     {
-      if (self->animation->decoded->len <= self->idx + 1) // currently on last decoded frame
-        {
-          GlyFrame *gly_frame;
-          GdkPixbufGlycinFrame frame;
+      self->time += g_array_index (self->animation->decoded, GdkPixbufGlycinFrame, self->idx).delay;
 
-          gly_frame = gly_image_next_frame (self->animation->image, NULL);
-          if (!gly_frame)
+      if (self->idx + 1 < self->animation->decoded->len)
+        {
+          self->idx++;
+        }
+      else if (g_array_index (self->animation->decoded, GdkPixbufGlycinFrame, self->idx).is_last_frame)
+        {
+          if (self->idx == 0)
             return FALSE;
 
-          frame.pixbuf = convert_glycin_frame_to_pixbuf (gly_frame);
-          frame.delay = gly_frame_get_delay (gly_frame);
-          g_array_append_val (self->animation->decoded, frame);
-
-          g_object_unref (gly_frame);
+          self->idx = 0;
         }
+      else
+        {
+          GlyFrameRequest *request;
+          GlyFrame *gly_frame;
+          GError *error = NULL;
 
-      self->time += g_array_index (self->animation->decoded, GdkPixbufGlycinFrame, self->idx).delay;
-      self->idx++;
-      retval = TRUE;
+          request = gly_frame_request_new ();
+          gly_frame_request_set_scale (request, self->animation->width, self->animation->height);
+          gly_frame_request_set_loop_animation (request, FALSE);
+          gly_frame = gly_image_get_specific_frame (self->animation->image,
+                                                    request,
+                                                    &error);
+          g_object_unref (request);
+
+          if (gly_frame)
+            {
+              GdkPixbufGlycinFrame frame;
+
+              frame.pixbuf = convert_glycin_frame_to_pixbuf (gly_frame);
+              frame.delay = gly_frame_get_delay (gly_frame);
+              frame.is_last_frame = FALSE;
+              g_array_append_val (self->animation->decoded, frame);
+              g_object_unref (gly_frame);
+
+              self->idx++;
+            }
+          else if (g_error_matches (error, GLY_LOADER_ERROR, GLY_LOADER_ERROR_NO_MORE_FRAMES))
+            {
+              g_array_index (self->animation->decoded, GdkPixbufGlycinFrame, self->idx).is_last_frame = TRUE;
+
+              g_error_free (error);
+              self->idx = 0;
+
+              return FALSE;
+            }
+          else
+            {
+              g_error_free (error);
+              return FALSE;
+            }
+        }
     }
 
-  return retval;
+  return TRUE;
 }
 G_GNUC_END_IGNORE_DEPRECATIONS
 
@@ -354,7 +381,6 @@ G_GNUC_END_IGNORE_DEPRECATIONS
 
   self->animation = NULL;
   self->idx = 0;
-  self->on_last_frame = FALSE;
 }
 
 static GdkPixbufGlycinAnimation *
@@ -381,12 +407,13 @@ static GdkPixbuf *
 load_pixbuf_with_glycin (GFile                    *file,
                          GdkPixbufModuleSizeFunc   size_func,
                          gpointer                  user_data,
+                         GdkPixbufAnimation      **animation,
                          GError                  **error)
 {
   GlyLoader *loader;
   GlyImage *image;
   GlyFrameRequest *request = NULL;
-  GlyFrame *frame = NULL;
+  GlyFrame *gly_frame = NULL;
   GdkPixbuf *pixbuf = NULL;
   GError *local_error = NULL;
   int width, height;
@@ -413,13 +440,18 @@ load_pixbuf_with_glycin (GFile                    *file,
     size_func (&width, &height, user_data);
 
   request = gly_frame_request_new ();
-  gly_frame_request_set_scale (request, width, height);
 
-  frame = gly_image_get_specific_frame (image, request, &local_error);
-  if (!frame)
+  /* FIXME: Handle zero-size request and abort loading */
+  if (width > 0 && height > 0)
+    gly_frame_request_set_scale (request, width, height);
+
+  gly_frame = gly_image_get_specific_frame (image, request, &local_error);
+  if (!gly_frame)
     goto done;
 
-  pixbuf = convert_glycin_frame_to_pixbuf (frame);
+  pixbuf = convert_glycin_frame_to_pixbuf (gly_frame);
+  width = gdk_pixbuf_get_width (pixbuf);
+  height = gdk_pixbuf_get_height (pixbuf);
 
   g_snprintf (value, sizeof (value), "%u",
               gly_image_get_transformation_orientation (image));
@@ -439,11 +471,27 @@ load_pixbuf_with_glycin (GFile                    *file,
       g_strfreev (keys);
     }
 
+  if (animation && gly_frame_get_delay (gly_frame) != 0)
+    {
+      GdkPixbufGlycinFrame frame;
+      GdkPixbufGlycinAnimation *anim;
+
+      anim = gdk_pixbuf_glycin_animation_new (image, width, height);
+      frame.pixbuf = g_object_ref (pixbuf);
+      frame.delay = gly_frame_get_delay (gly_frame);
+      frame.is_last_frame = FALSE;
+      g_array_append_val (anim->decoded, frame);
+
+      *animation = (GdkPixbufAnimation *) anim;
+    }
+  else if (animation)
+    *animation = NULL;
+
 done:
   g_clear_object (&loader);
   g_clear_object (&image);
   g_clear_object (&request);
-  g_clear_object (&frame);
+  g_clear_object (&gly_frame);
 
   if (g_error_matches (local_error, GLY_LOADER_ERROR, GLY_LOADER_ERROR_FAILED))
     {
@@ -465,97 +513,6 @@ done:
   return pixbuf;
 }
 
-static GdkPixbufAnimation *
-load_animation_with_glycin (GFile                    *file,
-                            GdkPixbufModuleSizeFunc   size_func,
-                            gpointer                  user_data,
-                            GError                  **error)
-{
-  GlyLoader *loader;
-  GlyImage *image;
-  GlyFrameRequest *request = NULL;
-  GlyFrame *gly_frame = NULL;
-  GdkPixbufAnimation *animation = NULL;
-  GError *local_error = NULL;
-  int width, height;
-  GdkPixbuf *pixbuf = NULL;
-
-  loader = gly_loader_new (file);
-
-  if (should_run_unsandboxed ())
-    gly_loader_set_sandbox_selector (loader, GLY_SANDBOX_SELECTOR_NOT_SANDBOXED);
-
-  gly_loader_set_accepted_memory_formats (loader, GLY_MEMORY_SELECTION_R8G8B8A8 |
-                                                  GLY_MEMORY_SELECTION_R8G8B8);
-
-  gly_loader_set_apply_transformations (loader, FALSE);
-
-  image = gly_loader_load (loader, &local_error);
-  if (!image)
-    goto done;
-
-  width = gly_image_get_width (image);
-  height = gly_image_get_height (image);
-  if (size_func)
-    size_func (&width, &height, user_data);
-
-  request = gly_frame_request_new ();
-  gly_frame_request_set_scale (request, width, height);
-
-  gly_frame = gly_image_get_specific_frame (image, request, &local_error);
-  if (!gly_frame)
-    goto done;
-
-  pixbuf = convert_glycin_frame_to_pixbuf (gly_frame);
-
-  if (gly_frame_get_delay (gly_frame) != 0)
-    {
-      GdkPixbufGlycinFrame frame;
-      GdkPixbufGlycinAnimation *gly_anim;
-
-      gly_anim = gdk_pixbuf_glycin_animation_new (image, width, height);
-      frame.pixbuf = g_steal_pointer (&pixbuf);
-      frame.delay = gly_frame_get_delay (gly_frame);
-
-      g_array_append_val (gly_anim->decoded, frame);
-
-      animation = (GdkPixbufAnimation *) gly_anim;
-    }
-  else
-    {
-
-G_GNUC_BEGIN_IGNORE_DEPRECATIONS
-      animation = gdk_pixbuf_non_anim_new (pixbuf);
-G_GNUC_END_IGNORE_DEPRECATIONS
-    }
-
-done:
-  g_clear_object (&loader);
-  g_clear_object (&image);
-  g_clear_object (&request);
-  g_clear_object (&gly_frame);
-  g_clear_object (&pixbuf);
-
-  if (g_error_matches (local_error, GLY_LOADER_ERROR, GLY_LOADER_ERROR_FAILED))
-    {
-      g_set_error_literal (error, GDK_PIXBUF_ERROR, GDK_PIXBUF_ERROR_CORRUPT_IMAGE,
-                           local_error->message);
-      g_clear_error (&local_error);
-    }
-  else if (g_error_matches (local_error, GLY_LOADER_ERROR, GLY_LOADER_ERROR_UNKNOWN_IMAGE_FORMAT))
-    {
-      g_set_error_literal (error, GDK_PIXBUF_ERROR, GDK_PIXBUF_ERROR_UNKNOWN_TYPE,
-                           local_error->message);
-      g_clear_error (&local_error);
-    }
-  else if (local_error)
-    {
-      g_propagate_error (error, local_error);
-    }
-
-  return animation;
-}
-
 /* }}} */
 /* {{{ Vfuncs */
 
@@ -616,7 +573,7 @@ gdk_pixbuf__glycin_image_load (FILE *f, GError **error)
   if (!file)
     return NULL;
 
-  pixbuf = load_pixbuf_with_glycin (file, NULL, NULL, error);
+  pixbuf = load_pixbuf_with_glycin (file, NULL, NULL, NULL, error);
   g_object_unref (file);
 
   return pixbuf;
@@ -636,30 +593,46 @@ gdk_pixbuf__glycin_image_stop_load (gpointer   data,
 
   if (context->all_ok)
     {
+      GdkPixbuf *pixbuf;
       GdkPixbufAnimation *animation;
 
-      animation = load_animation_with_glycin (context->file,
-                                              context->size_func,
-                                              context->user_data,
-                                              error);
-      if (animation)
+      pixbuf = load_pixbuf_with_glycin (context->file,
+                                        context->size_func,
+                                        context->user_data,
+                                        &animation,
+                                        error);
+      if (pixbuf)
         {
-          GdkPixbuf *pixbuf;
+          GBytes *bytes;
+          const guchar *gly_data;
+          gsize len;
 
-G_GNUC_BEGIN_IGNORE_DEPRECATIONS
-          pixbuf = gdk_pixbuf_animation_get_static_image (animation);
-G_GNUC_END_IGNORE_DEPRECATIONS
+          bytes = gdk_pixbuf_read_pixel_bytes (pixbuf);
+          gly_data = g_bytes_get_data (bytes, &len);
+          g_assert (gdk_pixbuf_read_pixels (pixbuf) == gly_data);
+
+          (* context->prepared_func) (pixbuf, animation, context->user_data);
+
+          if (gdk_pixbuf_read_pixels (pixbuf) != gly_data)
+            {
+              guchar *pixbuf_data;
+
+              g_warning ("pixbuf was mutated in the prepare callback");
+
+              pixbuf_data = gdk_pixbuf_get_pixels (pixbuf);
+              memcpy (pixbuf_data, gly_data, len);
+            }
+          g_bytes_unref (bytes);
 
-          (* context->prepared_func) (pixbuf,
-                                      animation,
-                                      context->user_data);
           (* context->updated_func) (pixbuf,
                                      0, 0,
                                      gdk_pixbuf_get_width (pixbuf),
                                      gdk_pixbuf_get_height (pixbuf),
                                      context->user_data);
 
-          g_object_unref (animation);
+          if (animation)
+            g_object_unref (animation);
+          g_object_unref (pixbuf);
           retval = TRUE;
         }
     }
@@ -810,15 +783,26 @@ gdk_pixbuf__glycin_image_load_animation (FILE    *f,
                                          GError **error)
 {
   GFile *file;
-  GdkPixbufAnimation *animation;
+  GdkPixbufAnimation *animation = NULL;
+  GdkPixbuf *pixbuf;
 
   file = g_file_from_file (f, error);
   if (!file)
     return NULL;
 
-  animation = load_animation_with_glycin (file, NULL, NULL, error);
+  pixbuf = load_pixbuf_with_glycin (file, NULL, NULL, &animation, error);
   g_object_unref (file);
 
+  if (!pixbuf)
+    return NULL;
+
+  if (!animation)
+G_GNUC_BEGIN_IGNORE_DEPRECATIONS
+    animation = gdk_pixbuf_non_anim_new (pixbuf);
+G_GNUC_END_IGNORE_DEPRECATIONS
+
+  g_object_unref (pixbuf);
+
   return animation;
 }
 
