@@ -507,7 +507,7 @@ pixbuf_create_from_xpm (const gchar * (*get_buf) (enum buf_op op, gpointer handl
                                      _("Invalid XPM header"));
 		return NULL;
 	}
-	if (cpp <= 0 || cpp >= 32) {
+	if (cpp <= 0 || cpp >= 32 || w >= G_MAXINT / cpp) {
                 g_set_error_literal (error,
                                      GDK_PIXBUF_ERROR,
                                      GDK_PIXBUF_ERROR_CORRUPT_IMAGE,
@@ -698,8 +698,8 @@ gdk_pixbuf__xpm_image_load_xpm_data (const gchar **data)
 typedef struct _XPMContext XPMContext;
 struct _XPMContext
 {
-       GdkPixbufModulePreparedFunc prepare_func;
-       GdkPixbufModuleUpdatedFunc update_func;
+       GdkPixbufModulePreparedFunc prepared_func;
+       GdkPixbufModuleUpdatedFunc updated_func;
        gpointer user_data;
 
        gchar *tempname;
@@ -715,17 +715,21 @@ struct _XPMContext
  */
 static gpointer
 gdk_pixbuf__xpm_image_begin_load (GdkPixbufModuleSizeFunc size_func,
-                                  GdkPixbufModulePreparedFunc prepare_func,
-                                  GdkPixbufModuleUpdatedFunc update_func,
+                                  GdkPixbufModulePreparedFunc prepared_func,
+                                  GdkPixbufModuleUpdatedFunc updated_func,
                                   gpointer user_data,
                                   GError **error)
 {
        XPMContext *context;
        gint fd;
 
+       g_assert (size_func != NULL);
+       g_assert (prepared_func != NULL);
+       g_assert (updated_func != NULL);
+
        context = g_new (XPMContext, 1);
-       context->prepare_func = prepare_func;
-       context->update_func = update_func;
+       context->prepared_func = prepared_func;
+       context->updated_func = updated_func;
        context->user_data = user_data;
        context->all_okay = TRUE;
        fd = g_file_open_tmp ("gdkpixbuf-xpm-tmp.XXXXXX", &context->tempname,
@@ -761,16 +765,14 @@ gdk_pixbuf__xpm_image_stop_load (gpointer data,
                pixbuf = gdk_pixbuf__xpm_image_load (context->file, error);
 
                if (pixbuf != NULL) {
-		       if (context->prepare_func)
-			       (* context->prepare_func) (pixbuf,
-							  NULL,
-							  context->user_data);
-		       if (context->update_func)
-			       (* context->update_func) (pixbuf,
-							 0, 0,
-							 gdk_pixbuf_get_width (pixbuf),
-							 gdk_pixbuf_get_height (pixbuf),
-							 context->user_data);
+		       (* context->prepared_func) (pixbuf,
+						   NULL,
+						   context->user_data);
+		       (* context->updated_func) (pixbuf,
+						  0, 0,
+						  gdk_pixbuf_get_width (pixbuf),
+						  gdk_pixbuf_get_height (pixbuf),
+						  context->user_data);
                        g_object_unref (pixbuf);
 
                        retval = TRUE;
