@@ -21,8 +21,8 @@
 #include <math.h>
 #include <glib.h>
 
+#include "../fallback-c89.c"
 #include "pixops.h"
-#include "pixops-internal.h"
 
 #define SUBSAMPLE_BITS 4
 #define SUBSAMPLE (1 << SUBSAMPLE_BITS)
@@ -836,36 +836,6 @@ composite_line_22_4a4 (int *weights, int n_x, int n_y,
   return dest;
 }
 
-#ifdef USE_MMX
-static guchar *
-composite_line_22_4a4_mmx_stub (int *weights, int n_x, int n_y, guchar *dest,
-				int dest_x, guchar *dest_end,
-				int dest_channels, int dest_has_alpha,
-				guchar **src, int src_channels,
-				gboolean src_has_alpha, int x_init,
-				int x_step, int src_width, int check_size,
-				guint32 color1, guint32 color2)
-{
-  guint32 mmx_weights[16][8];
-  int j;
-
-  for (j=0; j<16; j++)
-    {
-      mmx_weights[j][0] = 0x00010001 * (weights[4*j] >> 8);
-      mmx_weights[j][1] = 0x00010001 * (weights[4*j] >> 8);
-      mmx_weights[j][2] = 0x00010001 * (weights[4*j + 1] >> 8);
-      mmx_weights[j][3] = 0x00010001 * (weights[4*j + 1] >> 8);
-      mmx_weights[j][4] = 0x00010001 * (weights[4*j + 2] >> 8);
-      mmx_weights[j][5] = 0x00010001 * (weights[4*j + 2] >> 8);
-      mmx_weights[j][6] = 0x00010001 * (weights[4*j + 3] >> 8);
-      mmx_weights[j][7] = 0x00010001 * (weights[4*j + 3] >> 8);
-    }
-
-  return _pixops_composite_line_22_4a4_mmx (mmx_weights, dest, src[0], src[1],
-					    x_step, dest_end, x_init);
-}
-#endif /* USE_MMX */
-
 static void
 composite_pixel_color (guchar *dest, int dest_x, int dest_channels,
 		       int dest_has_alpha, int src_has_alpha, int check_size,
@@ -979,44 +949,6 @@ composite_line_color (int *weights, int n_x, int n_y, guchar *dest,
   return dest;
 }
 
-#ifdef USE_MMX
-static guchar *
-composite_line_color_22_4a4_mmx_stub (int *weights, int n_x, int n_y,
-				      guchar *dest, int dest_x,
-				      guchar *dest_end, int dest_channels,
-				      int dest_has_alpha, guchar **src,
-				      int src_channels, gboolean src_has_alpha,
-				      int x_init, int x_step, int src_width,
-				      int check_size, guint32 color1,
-				      guint32 color2)
-{
-  guint32 mmx_weights[16][8];
-  int check_shift = get_check_shift (check_size);
-  int colors[4];
-  int j;
-
-  for (j=0; j<16; j++)
-    {
-      mmx_weights[j][0] = 0x00010001 * (weights[4*j] >> 8);
-      mmx_weights[j][1] = 0x00010001 * (weights[4*j] >> 8);
-      mmx_weights[j][2] = 0x00010001 * (weights[4*j + 1] >> 8);
-      mmx_weights[j][3] = 0x00010001 * (weights[4*j + 1] >> 8);
-      mmx_weights[j][4] = 0x00010001 * (weights[4*j + 2] >> 8);
-      mmx_weights[j][5] = 0x00010001 * (weights[4*j + 2] >> 8);
-      mmx_weights[j][6] = 0x00010001 * (weights[4*j + 3] >> 8);
-      mmx_weights[j][7] = 0x00010001 * (weights[4*j + 3] >> 8);
-    }
-
-  colors[0] = (color1 & 0xff00) << 8 | (color1 & 0xff);
-  colors[1] = (color1 & 0xff0000) >> 16;
-  colors[2] = (color2 & 0xff00) << 8 | (color2 & 0xff);
-  colors[3] = (color2 & 0xff0000) >> 16;
-
-  return _pixops_composite_line_color_22_4a4_mmx (mmx_weights, dest, src[0],
-    src[1], x_step, dest_end, x_init, dest_x, check_shift, colors);
-}
-#endif /* USE_MMX */
-
 static void
 scale_pixel (guchar *dest, int dest_x, int dest_channels, int dest_has_alpha,
 	     int src_has_alpha, int check_size, guint32 color1, guint32 color2,
@@ -1024,11 +956,20 @@ scale_pixel (guchar *dest, int dest_x, int dest_channels, int dest_has_alpha,
 {
   if (src_has_alpha)
     {
-      if (a)
+      if (a == 0xff0000)
 	{
-	  dest[0] = r / a;
-	  dest[1] = g / a;
-	  dest[2] = b / a;
+	  /* Division by a constant is converted into a multiplication by an optimizing C compiler */
+	  dest[0] = r / 0xff0000;
+	  dest[1] = g / 0xff0000;
+	  dest[2] = b / 0xff0000;
+	  dest[3] = 0xff0000 >> 16;
+	}
+      else if (a)
+	{
+	  double a1 = 1.0 / a;
+	  dest[0] = r * a1;
+	  dest[1] = g * a1;
+	  dest[2] = b * a1;
 	  dest[3] = a >> 16;
 	}
       else
@@ -1057,8 +998,9 @@ scale_line (int *weights, int n_x, int n_y, guchar *dest, int dest_x,
 	    int x_step, int src_width, int check_size, guint32 color1,
 	    guint32 color2)
 {
-  int x = x_init;
+  gint64 x = x_init;
   int i, j;
+  const gint64 n_xy = n_x * n_y;
 
   while (dest < dest_end)
     {
@@ -1066,17 +1008,17 @@ scale_line (int *weights, int n_x, int n_y, guchar *dest, int dest_x,
       int *pixel_weights;
 
       pixel_weights = weights +
-        ((x >> (SCALE_SHIFT - SUBSAMPLE_BITS)) & SUBSAMPLE_MASK) * n_x * n_y;
+        ((x >> (SCALE_SHIFT - SUBSAMPLE_BITS)) & SUBSAMPLE_MASK) * n_xy;
 
       if (src_has_alpha)
 	{
 	  unsigned int r = 0, g = 0, b = 0, a = 0;
-	  for (i=0; i<n_y; i++)
+	  for (i = 0; i < n_y; i++)
 	    {
 	      guchar *q = src[i] + x_scaled * src_channels;
 	      int *line_weights  = pixel_weights + n_x * i;
 	      
-	      for (j=0; j<n_x; j++)
+	      for (j = 0; j < n_x; j++)
 		{
 		  unsigned int ta;
 		  
@@ -1090,11 +1032,20 @@ scale_line (int *weights, int n_x, int n_y, guchar *dest, int dest_x,
 		}
 	    }
 
-	  if (a)
+	  if (a == 0xff0000)
 	    {
-	      dest[0] = r / a;
-	      dest[1] = g / a;
-	      dest[2] = b / a;
+	      /* Division by a constant is converted into a multiplication by an optimizing C compiler */
+	      dest[0] = r / 0xff0000;
+	      dest[1] = g / 0xff0000;
+	      dest[2] = b / 0xff0000;
+	      dest[3] = 0xff0000 >> 16;
+	    }
+	  else if (a)
+	    {
+	      double a1 = 1.0 / a;
+	      dest[0] = r * a1;
+	      dest[1] = g * a1;
+	      dest[2] = b * a1;
 	      dest[3] = a >> 16;
 	    }
 	  else
@@ -1108,12 +1059,12 @@ scale_line (int *weights, int n_x, int n_y, guchar *dest, int dest_x,
       else
 	{
 	  unsigned int r = 0, g = 0, b = 0;
-	  for (i=0; i<n_y; i++)
+	  for (i = 0; i < n_y; i++)
 	    {
 	      guchar *q = src[i] + x_scaled * src_channels;
 	      int *line_weights  = pixel_weights + n_x * i;
 	      
-	      for (j=0; j<n_x; j++)
+	      for (j = 0; j < n_x; j++)
 		{
 		  unsigned int ta = line_weights[j];
 		  
@@ -1141,34 +1092,6 @@ scale_line (int *weights, int n_x, int n_y, guchar *dest, int dest_x,
   return dest;
 }
 
-#ifdef USE_MMX 
-static guchar *
-scale_line_22_33_mmx_stub (int *weights, int n_x, int n_y, guchar *dest,
-			   int dest_x, guchar *dest_end, int dest_channels,
-			   int dest_has_alpha, guchar **src, int src_channels,
-			   gboolean src_has_alpha, int x_init, int x_step,
-			   int src_width, int check_size, guint32 color1,
-			   guint32 color2)
-{
-  guint32 mmx_weights[16][8];
-  int j;
-
-  for (j=0; j<16; j++)
-    {
-      mmx_weights[j][0] = 0x00010001 * (weights[4*j] >> 8);
-      mmx_weights[j][1] = 0x00010001 * (weights[4*j] >> 8);
-      mmx_weights[j][2] = 0x00010001 * (weights[4*j + 1] >> 8);
-      mmx_weights[j][3] = 0x00010001 * (weights[4*j + 1] >> 8);
-      mmx_weights[j][4] = 0x00010001 * (weights[4*j + 2] >> 8);
-      mmx_weights[j][5] = 0x00010001 * (weights[4*j + 2] >> 8);
-      mmx_weights[j][6] = 0x00010001 * (weights[4*j + 3] >> 8);
-      mmx_weights[j][7] = 0x00010001 * (weights[4*j + 3] >> 8);
-    }
-
-  return _pixops_scale_line_22_33_mmx (mmx_weights, dest, src[0], src[1],
-				       x_step, dest_end, x_init);
-}
-#endif /* USE_MMX */
 
 static guchar *
 scale_line_22_33 (int *weights, int n_x, int n_y, guchar *dest, int dest_x,
@@ -1411,7 +1334,7 @@ pixops_process (guchar         *dest_buf,
    */
 #define MYDIV(a,b) ((a) > 0 ? (a) / (b) : ((a) - (b) + 1) / (b))    /* Division so that -1/5 = -1 */
 
-  run_end_x = (((src_width - filter->x.n + 1) << SCALE_SHIFT) - scaled_x_offset);
+  run_end_x = (((src_width - (gint64) filter->x.n + 1) << SCALE_SHIFT) - scaled_x_offset);
   run_end_index = MYDIV (run_end_x + x_step - 1, x_step) - render_x0;
   run_end_index = MIN (run_end_index, render_x1 - render_x0);
 
@@ -1793,6 +1716,8 @@ need_to_prescale (double           scale_x,
   case PIXOPS_INTERP_NEAREST:
     /* Doesn't need the optimization */
     return FALSE;
+  default:
+    g_assert_not_reached ();
   }
 
   /* Limit the number of filters created by make_filter_table(). */
@@ -1907,10 +1832,6 @@ _pixops_composite_color_real (guchar          *dest_buf,
   PixopsLineFunc line_func;
   guchar *tmp_buf = NULL;
   
-#ifdef USE_MMX
-  gboolean found_mmx = _pixops_have_mmx ();
-#endif
-
   g_return_if_fail (!(dest_channels == 3 && dest_has_alpha));
   g_return_if_fail (!(src_channels == 3 && src_has_alpha));
 
@@ -1936,16 +1857,9 @@ _pixops_composite_color_real (guchar          *dest_buf,
   
   filter.overall_alpha = overall_alpha / 255.;
   if (!make_weights (&filter, interp_type, scale_x, scale_y))
-    return;
+    goto free_tmp;
 
-#ifdef USE_MMX
-  if (filter.x.n == 2 && filter.y.n == 2 &&
-      dest_channels == 4 && src_channels == 4 &&
-      src_has_alpha && !dest_has_alpha && found_mmx)
-    line_func = composite_line_color_22_4a4_mmx_stub;
-  else
-#endif
-    line_func = composite_line_color;
+  line_func = composite_line_color;
   
   pixops_process (dest_buf, render_x0, render_y0, render_x1, render_y1,
 		  dest_rowstride, dest_channels, dest_has_alpha,
@@ -1955,8 +1869,8 @@ _pixops_composite_color_real (guchar          *dest_buf,
 
   g_free (filter.x.weights);
   g_free (filter.y.weights);
-  if (tmp_buf)
-    g_free (tmp_buf);
+free_tmp:
+  g_free (tmp_buf);
 }
 
 void
@@ -2068,10 +1982,6 @@ _pixops_composite_real (guchar          *dest_buf,
   PixopsLineFunc line_func;
   guchar *tmp_buf = NULL;
   
-#ifdef USE_MMX
-  gboolean found_mmx = _pixops_have_mmx ();
-#endif
-
   g_return_if_fail (!(dest_channels == 3 && dest_has_alpha));
   g_return_if_fail (!(src_channels == 3 && src_has_alpha));
 
@@ -2100,18 +2010,11 @@ _pixops_composite_real (guchar          *dest_buf,
   
   filter.overall_alpha = overall_alpha / 255.;
   if (!make_weights (&filter, interp_type, scale_x, scale_y))
-    return;
+    goto free_tmp;
 
   if (filter.x.n == 2 && filter.y.n == 2 && dest_channels == 4 &&
       src_channels == 4 && src_has_alpha && !dest_has_alpha)
-    {
-#ifdef USE_MMX
-      if (found_mmx)
-	line_func = composite_line_22_4a4_mmx_stub;
-      else
-#endif	
-	line_func = composite_line_22_4a4;
-    }
+    line_func = composite_line_22_4a4;
   else
     line_func = composite_line;
   
@@ -2123,8 +2026,8 @@ _pixops_composite_real (guchar          *dest_buf,
 
   g_free (filter.x.weights);
   g_free (filter.y.weights);
-  if (tmp_buf)
-    g_free (tmp_buf);
+free_tmp:
+  g_free (tmp_buf);
 }
 
 void
@@ -2488,10 +2391,6 @@ _pixops_scale_real (guchar        *dest_buf,
   PixopsLineFunc line_func;
   guchar *tmp_buf = NULL;	/* Temporary image for two-step scaling */
 
-#ifdef USE_MMX
-  gboolean found_mmx = _pixops_have_mmx ();
-#endif
-
   g_return_if_fail (!(dest_channels == 3 && dest_has_alpha));
   g_return_if_fail (!(src_channels == 3 && src_has_alpha));
   g_return_if_fail (!(src_has_alpha && !dest_has_alpha));
@@ -2516,17 +2415,10 @@ _pixops_scale_real (guchar        *dest_buf,
   
   filter.overall_alpha = 1.0;
   if (!make_weights (&filter, interp_type, scale_x, scale_y))
-    return;
+    goto free_tmp;
 
   if (filter.x.n == 2 && filter.y.n == 2 && dest_channels == 3 && src_channels == 3)
-    {
-#ifdef USE_MMX
-      if (found_mmx)
-	line_func = scale_line_22_33_mmx_stub;
-      else
-#endif
-	line_func = scale_line_22_33;
-    }
+    line_func = scale_line_22_33;
   else
     line_func = scale_line;
   
@@ -2538,7 +2430,8 @@ _pixops_scale_real (guchar        *dest_buf,
 
   g_free (filter.x.weights);
   g_free (filter.y.weights);
-  g_clear_pointer (&tmp_buf, g_free);
+free_tmp:
+  g_free (tmp_buf);
 }
 
 void
